import java.util.Random;
import java.util.Date;

/**
 * This class is the number picker  It picks a random number between the min, and 
 * max fields (inclusive) and will let you know if you guess the right number.
 * 
 * @author Barbara Ericson ericson@cc.gatech.edu
 */
public class NumberPicker 
{
  
  ////////////////////// Fields //////////////////////////////////
  
  /** The picked number */
  private int pickedNumber = 0;
  
  /** The random number generator to use */
  private Random randomPicker = null;
  
  /** The maximum value that can be picked */
  private int max;
  
  /** The minimum value that can be picked */
  private int min; 
  
  /** The default minimum value */
  public static final int DEFAULT_MIN = 1;
  
  /** The default maximum value */
  public static final int DEFAULT_MAX = 10;
  
  /** The default seed is based on the current time */
  public static final long DEFAULT_SEED = new Date().getTime();
  
  /////////////////////// Constructors ////////////////////////////
  
  /**
   * Constructor that takes no arguments. It will set the min, max, and 
   * seed to the default values
   */
  public NumberPicker()
  {
    initialize(DEFAULT_MIN,DEFAULT_MAX,DEFAULT_SEED);
  }
  
  /**
   * Constructor that takes a seed for the random number generator
   * @param seed the seed for the random number generator
   */
  public NumberPicker(long seed)
  {
    /* call the constructor that takes min, max, and seed
     * and just use default values for min and max 
     */
    initialize(DEFAULT_MIN,DEFAULT_MAX,seed);
  }
  
  /**
   * Constructor that takes the min, max, and seed 
   * @param min the min to use
   * @param max the max to use
   * @param seed the seed for the random number generator
   */
  public NumberPicker(int min, int max, long seed)
  {
    initialize(min,max,seed);
  }
  
  ////////////////////////// Methods ///////////////////////////////////
  
  /**
   * Method to get the minimum value that can be picked
   * @return the minimum value that can be picked
   */
  public int getMin()
  { return min; }
  
  /**
   * Method to get the maximum value that can be picked
   * @return the maximum value that can be picked
   */
  public int getMax()
  { return max; }
  
  /**
   * Method to set the minimum value
   * @param value the new minimum to use
   */
  public void setMin(int value)
  { min = value; }
  
  /**
   * Method to set the maximum value
   * @param value the new maximum to use
   */
  public void setMax(int value)
  { max = value; }
  
  /**
   * Method to initialize the fields for this object
   * @param min the minimum value that can be picked
   * @param max the maximum value that can be picked
   * @param seed the seed for the random number generator
   */
  private void initialize(int min, int max, long seed)
  {
    this.min = min;
    this.max = max;
    
    // get a random number generator with the specified seed
    randomPicker = new Random(seed);
    
    // pick a random number
    pickANumber();
  }
  
  /**
   * Method to pick a random number between the min and max field values
   * (inclusive)
   */
  public void pickANumber()
  {
     // random picker will return a number from 0 to the input - 1
    int randomNum = randomPicker.nextInt(max + 1 - min);
    
    // get the actual picked number in the range
    pickedNumber = randomNum + min;
    
  }
  
  /**
   * Check if the given guess is equal to the picked number
   * @param guess the guessed number
   * @return true if the guess is correct else false
   */
  public boolean isCorrect(int guess)
  {
    return (guess == pickedNumber);
  }
    
}
