/*
 * GuessWhoGame.java
 * 
 * Copyright (c) 2004 ICEGT, Jeremy Goccks
 */
package edu.gatech.icegt.guesswho;

import java.awt.Container;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.ImageObserver;
import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.Properties;

import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;

/**
 * The main class for the GuessWho game.
 * 
 * @author Jeremy Goecks
 * @version 0.1
 */
public class GuessWhoGame extends JFrame 
	implements ActionListener, GuessWhoGameConstants {
	
	/**
	 * Filename filer that accepts only files that end with the GuessWhoGame
	 * extension.
	 */
	protected static class GuessWhoFilenameFilter implements FilenameFilter {
		/**
		 * @see java.io.FilenameFilter#accept(java.io.File, java.lang.String)
		 */
		public boolean accept(File dir, String name) {
			return name.endsWith(PERSON_FILE_EXT);
		} // end accept()
	} // end class GuessWhoFilenameFilter()
	
	/**
	 * Read Person objects from files.
	 * @param personFilesDir directory where person files are
	 */
	public static Person[] readPersonFiles(String personFilesDir) {
		// Get files in directory with the GuessWho extension.
		File directory = new File(personFilesDir);
		File[] personFiles = directory.listFiles(new GuessWhoFilenameFilter());
		
		// Read Person files.
		Person[] people = new Person[personFiles.length];
		for (int i = 0; i < people.length; i++) {
			try { people[i] = Person.readFromFile(personFiles[i]); }
			catch (IOException ioe) {
				ioe.printStackTrace();
			} // end catch
		} // end for 
		
		return people;
	} // end readPersonFiles()
	
	/**
	 * Returns an ImageIcon with a scaled image.
	 * @param imageFile location of image file
	 * @param width image's scaled width
	 * @param observer image's observer
	 */
	protected static ImageIcon getImageIconWithScaledImage(File imageFile,
														   int scaledWidth,
														   ImageObserver observer) {
		return getImageIconWithScaledImage(imageFile.getAbsoluteFile().toString(), 
											scaledWidth, observer);
	} // end getImageIconWithScaledImage()
	
	/**
	 * Returns an ImageIcon with a scaled image.
	 * @param imageFile location of image file
	 * @param width image's scaled width
	 * @param observer image's observer
	 */
	protected static ImageIcon getImageIconWithScaledImage(String imageFile,
														   int scaledWidth,
														   ImageObserver observer) {
		// Create icon.
		ImageIcon icon = new ImageIcon(imageFile);
		
		// Scale image.
		Image image = icon.getImage();
		int width = image.getWidth(observer), height = image.getHeight(observer);
		double scaleFactor = scaledWidth/(double)width;
		int 
			newWidth = (int)Math.round(width*scaleFactor), 
			newHeight = (int)Math.round(height*scaleFactor);
		Image scaledImage = 
			image.getScaledInstance(newWidth, newHeight, Image.SCALE_DEFAULT);;
		
		// Set image and return.
		icon.setImage(scaledImage);
		return icon;
	} // end getImageIconWithScaledImage()
	
	/** Button that opens the frame to add people. */
	protected JButton addPeopleButton;
	
	/** Button that opens the frame to play the game. */
	protected JButton playGameButton;
	
	/** 'No picture' file. */
	protected String noPicFile;
	
	/** Directory where person files live. */
	protected String personFilesDir;
	
	/** Directory where person images live. */
	protected String personImagesDir;
	
	/**
	 * Create frame. Frame has two buttons: one to add people, one to play the 
	 * game.
	 */
	public GuessWhoGame() {
		//
		// Read properties file.
		//
		Properties applicationProps = new Properties();
		File propFile = new File(PROP_FILE_LOCATION);
		boolean success = false;
		try {
			FileInputStream propFileIn = new FileInputStream(propFile);
			applicationProps.load(propFileIn);
			success = true;
		} // end try
		catch (IOException ioe) {
			ioe.printStackTrace();
		} // end catch
		
		//
		// If we successfully got properties, continue creating the application.
		//
		if (success) {
			//
			// Extract/set properties.
			//
			noPicFile = applicationProps.getProperty(NO_PIC_FILE_PROP);
			personFilesDir = applicationProps.getProperty(PERSON_FILES_DIR);
			personImagesDir = applicationProps.getProperty(PERSON_IMAGES_DIR);
			String noMatchFile = applicationProps.getProperty(NO_MATCH_FILE);
			
			// Set "no match" file, person images directory for PersonPanel.
			PersonPanel.setNoMatchImageFile(noMatchFile);
			PersonPanel.setPersonImagesDir(personImagesDir);
			
			//
			// Create two buttons to lauch application frames.
			//
			Container c = getContentPane();
			c.setLayout(new BoxLayout(c, BoxLayout.Y_AXIS));
			addPeopleButton = new JButton("Add people to game");
			addPeopleButton.addActionListener(this);
			c.add(addPeopleButton);
			playGameButton = new JButton("Play Game");
			playGameButton.addActionListener(this);
			c.add(playGameButton);
			
			//
			// Set frame attributes.
			//
			setTitle("Guess Who Game");
			setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			
			//
			// Pack and show.
			//
			pack();
			show();
		} // end if
	} // end GuessWhoGame()
	
	/**
	 * Run the GuessWho game.
	 * @param args
	 */
	public static void main(String[] args) {
		new GuessWhoGame();
	} // end main()
	
	/**
	 * Take action depending on the button that user clicked on.
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		Object source = e.getSource();
		if (source == addPeopleButton) {
			//
			// Open frame to add people.
			//
			PersonInputFrame pif = 
				new PersonInputFrame(noPicFile, personFilesDir, personImagesDir);
			pif.pack();
			pif.show();
		} // end if
		else if (source == playGameButton) {
			//
			// Open frame to play game.
			//
			
			// Get people.
			Person[] people = readPersonFiles(personFilesDir);
			
			// Create game engine.
			GameEngine gameEngine = new GameEngine(people);
			
			// Create frame.
			PlayGameFrame pgf = new PlayGameFrame(gameEngine);
		} // end else if
	} // end actionPerformed()
} // end class GuessWhoGame
